-- colors
cBlack = color.new(0, 0, 0);
cWhite = color.new(255, 255, 255);

-- images
imgPlayer = {};
imgPlayer[1] = image.load("player1.png");
imgPlayer[2] = image.load("player2.png");
imgFrame = image.load("touchframe.png");
imgNoTouch = image.load("notouch.png");
imgButton = image.load("button.png");
imgBackdrop = image.load("bg.png");

-- game data
field = {};
current_player = 1;
scr_w = screen.width();
scr_h = screen.height();
winner = 0;
nTurns = 0;
touch_x = 0;
touch_y = 0;
endgame = false;

--[[
field structure:
1 2 3
4 5 6
7 8 9
]]

-- reset rows & cols
for i = 1, 9 do
	field[i] = 0;
end;

screen.clear();

function cls()
	
	imgBackdrop:draw(0, 0);
	
end;

function SwitchPlayer()
	
	if current_player == 1 then
		current_player = 2;
	else
		current_player = 1;
	end;
	
	nTurns = nTurns + 1;
	
end;

function UpdateInput()
	
	
	local x = touch_x;
	local y = touch_y;
	
	-- get field part touched
	local fx = 1;
	local fy = 0;
	
	if y > 80 then fy = 1; end;
	if y > 160 then fy = 2; end;
	
	if x > 80 then fx = 2; end;
	if x > 160 then fx = 3; end;
	
	local touched_part = fy * 3 + fx;
	
	if field[touched_part] == 0 then
		field[touched_part] = current_player;
		SwitchPlayer();
		imgFrame:draw((fx - 1) * 80, fy * 80);
		screen.update();
		os.sleep(50);
	else
		imgNoTouch:draw((fx - 1) * 80, fy * 80);
		screen.update();
		os.sleep(50);
	end;
	
end;

function GetFieldCoords(n)

	local x, y;
	
	x = 80 * ((n - 1) % 3);
	y = 80 * math.floor((n - 1) / 3);
	
	return x, y;

end;

function DrawButton(x, y, caption)

	imgButton:draw(x, y);
	text.draw(x, y + image.height(imgButton) / 2 - 10, caption, "center", image.width(imgButton));

end;

function DrawGame()

	local img;
	local w, h;
	local x, y;
	
	for i = 1, 9 do
		if field[i] ~= 0 then
			img = imgPlayer[field[i]];
			x, y = GetFieldCoords(i);
			w = image.width(img);
			h = image.height(img);
			img:draw(x + (40 - w / 2), y + (40 - h / 2));
		end;
	end;

	-- texts
	text.color(cBlack);
	text.size(20);
	
	text.draw(250, 10, "Player #" .. current_player);
	imgPlayer[current_player]:draw(250, 30);
	
	if winner ~= 0 then
		if winner > 0 then
			text.draw(250, 110, "Player #" .. winner .. " won!");
		else
			text.draw(250, 110, "Round draw!");
		end;
	end;
	
	-- buttons
	DrawButton(250, 200, "Exit");
	if winner ~= 0 then DrawButton(250, 160, "Reset"); end;
	
end;

function CheckGame()

	-- check rows
	if field[1] == field[2] and field[2] == field[3] then
		winner = field[1];
	end;
	
	if field[4] == field[5] and field[5] == field[6] then
		winner = field[4];
	end;
	
	if field[7] == field[8] and field[8] == field[9] then
		winner = field[7];
	end;
	
	-- check cols
	if field[1] == field[4] and field[4] == field[7] then
		winner = field[1];
	end;
	
	if field[2] == field[5] and field[5] == field[8] then
		winner = field[2];
	end;
	
	if field[3] == field[6] and field[6] == field[9] then
		winner = field[3];
	end;
	
	-- check diagonals
	if field[1] == field[5] and field[5] == field[9] then
		winner = field[1];
	end;	
	
	if field[3] == field[5] and field[5] == field[7] then
		winner = field[3];
	end;
		
	-- check draw
	if nTurns == 9 and winner == 0 then
		winner = -1;
	end;
		
end;

function IsInRect(x, y, rx, ry, w, h)

	if x >= rx and x <= (rx + w) and y >= ry and y <= (ry + h) then
		return true;
	else
		return false;
	end;

end;

function ResetGame()

	for i = 1, 9 do
		field[i] = 0;
	end;
	
	current_player = 1;
	winner = 0;
	nTurns = 0;
	
end;

function UpdateButtons()

	-- quit button
	if IsInRect(touch_x, touch_y, 250, 200, image.width(imgButton), image.height(imgButton)) == true then
		endgame = true;
	end;

	-- reset button
	if winner ~= 0 then
		if IsInRect(touch_x, touch_y, 250, 160, image.width(imgButton), image.height(imgButton)) == true then
			ResetGame();
		end;
	end;
	
end;

while true do
	
	cls();
	DrawGame();
	CheckGame();
	
	if control.read() == 1 then
	
		if (control.isTouch() == 1) and (touch.click() == 1) then
			touch_x, touch_y = touch.pos();
			if (winner == 0) and (touch_x <= 240) then UpdateInput(); end;
			UpdateButtons();
		end;
	
		if (control.isButton() == 1) and (button.home() == 1) and (button.click() == 1) then 
			break;
		end;
		
	end;
	
	screen.update();
	
	os.sleep(8);
	
	if endgame == true then break; end;
	
end;